'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''
import pygame
import data

from spritesheet import *
from board import *
from hero import *
from healthbar import *
from cactus import *
from icecreambeam import *
from fireball import *
from dragon import *

from pygame.locals import *

screen = pygame.display.set_mode((640, 640))
screen_rect = screen.get_rect()
clock = pygame.time.Clock()

def main():
	pygame.init()
	
	pygame.display.set_caption('Melty Twist')
	pygame.mouse.set_visible(0)
	title_screen = pygame.image.load(data.filepath('title_screen.png')).convert()
	screen.blit(title_screen, (0, 0))
	pygame.display.flip()
	
	if pygame.mixer:
		pygame.mixer.music.load(data.filepath("title_screen.mp3"))
		pygame.mixer.music.play(-1)
	
	while 1:
		for event in pygame.event.get():
			if event.type == KEYDOWN:
				game()
				return

def game():
	if not pygame.mixer:
		print "Warning, could not initialize sound!"
	else:
		pygame.mixer.music.load(data.filepath("oskari_wins_the_game.mp3"))
	
	spritesheet = Spritesheet()
	board = Board(spritesheet)
	board_group = pygame.sprite.RenderPlain(board)
	
	hero = Hero(spritesheet)
	hero_group = pygame.sprite.RenderPlain(hero)
	
	healthbar = HealthBar(spritesheet)
	healthbar_group = pygame.sprite.RenderPlain(healthbar)
	
	cactii = []
	cactus_group = pygame.sprite.RenderPlain()
	for i in range(10):
		cactii.append(Cactus(spritesheet))
		cactus_group.add(cactii[i])
	
	icecreambeam = IceCreamBeam(spritesheet)
	icecreambeam_group = pygame.sprite.RenderPlain(icecreambeam)
	
	fireball = Fireball(spritesheet)
	fireball_group = pygame.sprite.RenderPlain(fireball)

	dragon = Dragon(spritesheet)
	enemy_group = pygame.sprite.RenderPlain(dragon)

	board_group.draw(screen)
	if pygame.mixer:
		pygame.mixer.music.play(-1)

	while 1:
		clock.tick(60)
		direction = 0
		mx, my = hero.moving
		for event in pygame.event.get():
			if event.type == QUIT:
				return
			elif event.type == KEYDOWN:
				if event.key == K_UP:
					hero.direction = 0
					my = 1
				elif event.key == K_RIGHT:
					hero.direction = 1
					mx = 1
				elif event.key == K_DOWN:
					hero.direction = 2
					my = 1
				elif event.key == K_LEFT:
					hero.direction = 3
					mx = 1
				elif event.key == K_SPACE:
					if icecreambeam.bullets > 0:
						icecreambeam.shoot(hero.rect, hero.direction)
				elif event.key == K_ESCAPE:
					return	
			elif event.type == KEYUP:
				if event.key == K_UP or event.key == K_DOWN:
					mx, my = hero.moving
					my = 0
				elif event.key == K_LEFT or event.key == K_RIGHT:
					mx, my = hero.moving
					mx = 0

		hero.moving = (mx, my)
		hero_group.clear(screen, board.image)
		icecreambeam_group.clear(screen, board.image)
		cactus_group.clear(screen, board.image)
		healthbar_group.clear(screen, board.image)
		enemy_group.clear(screen, board.image)
		fireball_group.clear(screen, board.image)
		
		if hero.health >= 0: 
			cactus_group.update()
			cactus_group.draw(screen)
		
		if hero.health >= 0: 
			enemy_group.update(hero.rect.topleft, fireball)
			enemy_group.draw(screen)
		
		if hero.health >= 0: 
			fireball_group.update(0)
			fireball_group.draw(screen)
		
		if hero.health <= 0:
			screen.fill((0, 0, 0))
		
		hero_group.update()
		hero_group.draw(screen)
		
		if icecreambeam.shot == 1:
			icecreambeam.update(hero.bullets)
			icecreambeam_group.draw(screen)
		
		cactus_bullet_list = pygame.sprite.groupcollide(cactus_group, icecreambeam_group, False, False)
		for k, v in cactus_bullet_list.iteritems():
			k.freeze()
			icecreambeam.hide()
		cactus_player_list = pygame.sprite.groupcollide(cactus_group, hero_group, False, False)
		for k, v in cactus_player_list.iteritems():
			if k.frozen == 0:
				if hero.invincible_time == 0: 
					hero.health -= 10
					hero.ouch()
			else: 
				hero.health += 5
				hero.score += 100
				if hero.health > 100: hero.health = 100
				k.start()
		enemy_bullet_list = pygame.sprite.groupcollide(enemy_group, icecreambeam_group, False, False)
		for k, v in enemy_bullet_list.iteritems():
			if k.frozen == 0:
				k.shot()
				hero.bullets += 1
				icecreambeam.hide()
		enemy_player_list = pygame.sprite.groupcollide(enemy_group, hero_group, False, False)
		for k, v in enemy_player_list.iteritems():
			if k.frozen == 0:
				if hero.invincible_time == 0:
					hero.health -= 20
					hero.ouch()
			else:
				k.reset()
				hero.score += 500
				hero.health += 10
		player_enemy_bullet_list = pygame.sprite.groupcollide(fireball_group, hero_group, False, False)
		for k, v in player_enemy_bullet_list.iteritems():
			if hero.invincible_time == 0:
				hero.ouch()
				hero.health -= 20
				k.hide()
		
		if hero.health > 0:
			healthbar_group.update(hero.health)
			healthbar_group.draw(screen)
			
		if hero.health <= 0 and hero.animation < 120:
			hero.health = 0
			hero.start_melt(spritesheet)
			pygame.mixer.music.fadeout(2000)
			icecreambeam.kill()
			fireball.kill()
			for i in range(10):
				cactii[i].kill()
			dragon.kill()
		elif hero.health <= 0 and hero.animation >= 120:
			icecreambeam_group.empty()
			fireball_group.empty()
			screen.fill((0, 0, 0))
			font = pygame.font.Font(None, 36)
			text = font.render("Total score: %s" % int(hero.score), 1, (255, 0, 0))
			textpos = text.get_rect(centerx = screen.get_rect().width / 2, centery = screen.get_rect().height / 2)
			screen.blit(text, textpos)
			pygame.display.flip()
			pygame.time.delay(2000)
			return
		
		pygame.display.flip()
		hero.health -= 0.025